/* 
 * Xgetit - use anon FTP to get the file named in the primary X selection.
 *        - file reference formats: (1) host:path/file, (2) path/file@host.
 *        - pop up a window to show status of transfer.
 *        - written by NASA/JSC/GM2/Ken Jenks, with major portions of the
 *          code lifted from xpaste, which was written by John DiMarco at 
 *          the University of Toronto, CSRI.
 */
#include <stdio.h>
#include <strings.h>
#include <X11/StringDefs.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <varargs.h>

#define WIDTHPADDING 6
#define HEIGHTPADDING 6

#ifndef lint
static char rcsid[] = "$Header: xgetit.c,v 1.0 92/04/20 23:18:50 kcj Exp $";
#endif

void Xit();

static XtCallbackRec done[] = {
	{Xit, NULL},
	{NULL,NULL},
};

Widget CreateWidget();
void ConfigureWidget();

char *status_message;
char *buff;
char *full_ref;
char host_name[2048],path_name[2048],file_name[2048];
Widget root, fm, button;

main(argc, argv)
int argc;
char *argv[];
{
	FILE *fp, *fopen();

	int bytes;
	Dimension height, width;
	char *p, *b, status_msg_buff[2048];
	static XFontStruct *font;
	int dir, ascent, descent;
	XCharStruct all;

	status_message = status_msg_buff;
	*status_message = (char) NULL;

	root = XtInitialize(argv[0], "XGetit", NULL, 0, &argc, argv);

	/* 
	 * Need to convert PRIMARY selection to CUT_BUFFER0 because XFetchBytes
	 * only knows how to get contents of cut buffer 0.  I could have used
	 * the Xt selection functions to grab the PRIMARY selection directly, 
	 * but they're unnecessarily complicated. -- John
	 */

	XConvertSelection(XtDisplay(root), XA_PRIMARY, XA_CUT_BUFFER0, None, 
					 DefaultRootWindow(XtDisplay(root)), 
                                         CurrentTime);
	buff=XFetchBytes(XtDisplay(root), &bytes);

	/* 
	 * If there is no current selection, exit quietly. 
	 */
	if(NULL==buff) (void)exit(0);

        full_ref=strncpy(malloc(bytes+1),buff,bytes);
        full_ref[bytes]=(char)NULL;

	/* Now, I'll determine the host, path and file names from the
	   full reference.
	*/

	/* Case 1: full_ref = "host:path/file"
	*/
	if (index(full_ref,':')) {

	  strncpy(host_name,full_ref,index(full_ref,':')-full_ref);
	  strcpy(path_name,index(full_ref,':')+1);

	  /* Case 1a: Either path has a / in it or not.
	  */
	  if(rindex(path_name,'/')) {
	    strcpy(file_name,rindex(path_name,'/')+1);
	    *rindex(path_name,'/') = (char)NULL;

	  /* Case 1b: No / in path 
	  */
	  } else {
	    strcpy(file_name, path_name);
	    strcpy(path_name, ".");
	  }

	/* Case 2: full_ref = "path/file@host"
	*/
	} else if (index(full_ref,'@')) {

	  strncpy(path_name,full_ref,index(full_ref,'@')-full_ref);
	  strcpy(host_name,index(full_ref,'@')+1);

	  /* Case 2a: Either path has a / in it or not.
	  */
	  if(rindex(path_name,'/')) {
	    strcpy(file_name,rindex(path_name,'/')+1);
	    *rindex(path_name,'/') = (char)NULL;

	  /* Case 2b: No / in path 
	  */
	  } else {                   /* No / in path */
	    strcpy(file_name, path_name);
	    strcpy(path_name, ".");
	  }

	/* Case 3: full_ref = none of the above
	*/
	} else {
	  sprintf(status_message,
	    "Error: can't interpret file reference:\n%s\n\nUsage: xgetit provides rapid access to files using anonymous FTP.\nUse mouse to select a file reference, in form:\n   host:path/file    -or-   path/file@host\nthen run this program to get the file via anonymous FTP.\n",
	    full_ref);
	  goto show_status;
	}

	/* Remove extraneous CR from end of strings
	*/
	if(strlen(file_name))
	  if(file_name[strlen(file_name)-1] == 10)
	    file_name[strlen(file_name)-1] = (char) NULL;

	if(strlen(path_name))
	  if(path_name[strlen(path_name)-1] == 10)
	    path_name[strlen(path_name)-1] = (char) NULL;

	if(strlen(host_name))
	  if(host_name[strlen(host_name)-1] == 10)
	    host_name[strlen(host_name)-1] = (char) NULL;

	/* Open scratch file
	*/
	if((fp = fopen("xgetit_scratch_file","w")) == NULL) {
	  sprintf(status_message,"Error: can't open scratch file %s",
	    "xgetit_scratch_file");
	  goto show_status;
	}

	/* Build the commands needed to execute the anonymous FTP
	*/
	fprintf(fp,"#! /bin/csh\n");
	fprintf(fp,"/bin/echo \"Getting file %s from path %s at host %s...\"\n",
	  file_name,path_name,host_name);
	fprintf(fp,"/usr/bin/ftp -inv << EOF\n");
	fprintf(fp,"open %s\n",host_name);
	fprintf(fp,"\n");
	fprintf(fp,"user anonymous xgetit(kjenks@gothamcity.jsc.nasa.gov)\n");
	fprintf(fp,"binary\n"); /* Turn binary ON for all transfers */
	fprintf(fp,"prompt\n"); /* Turn prompt OFF for possible mget */
	if((*path_name != '.') || (strlen(path_name) != 1))
	  fprintf(fp,"cd %s\n",path_name);

	/* Do either mget or get, depening on existance of '*' in file_name
	*/
	if(index(file_name,'*'))
	  fprintf(fp,"mget %s\n", file_name); 
	else
	  fprintf(fp,"get %s\n", file_name); 

	fprintf(fp,"quit\n");
	fprintf(fp,"EOF\n");
	fclose(fp);

	/* Perform the FTP and capture the results
	*/
	system("sh xgetit_scratch_file > xgetit_error_file");

	/* Read any resulting error messages
	*/
	if((fp = fopen("xgetit_error_file","r")) == NULL) {
	  sprintf(status_message,"Warning: can't read error file %s",
	    "xgetit_error_file");
	  goto show_status;
	}

	status_message = status_msg_buff;
	while(feof(fp) == NULL)
	  *(status_message++) = fgetc(fp);
	status_message = status_msg_buff;

	fclose(fp);

	system("rm xgetit_scratch_file xgetit_error_file > /dev/null");

show_status:
 
	/* If there is no message to be displayed, exit quietly. 
	 */
	if(status_message == NULL) (void)exit(0);
	if(strlen(status_message) == 0) (void)exit(0);

	/*
	 * Compute the extents of the text. The eventual width of the text widget
	 * will be the length (in pixels) of the longest line. Variable width
	 * fonts are handled correctly. The eventual height of the text widget
	 * will be the sum of the heights of the lines in the selection. Yes, 
	 * computing the height for a single line and multiplying by the number
	 * of lines would be more efficient. But there's no server roundtrip
	 * required by XTextExtents, so the cost isn't high. And who knows: maybe
	 * we'll see variable height fonts? 
	 */

	{
		/* Get the default font */
		static XtResource resources[] = {
			{"font", XtCFont, XtRFontStruct, sizeof(XFontStruct *),
			(Cardinal)&font, XtRString, "fixed"},
		};
		XtGetApplicationResources(root, (caddr_t)NULL, resources,
							 
XtNumber(resources), NULL, (Cardinal)0);
	}
	width=0;
	height=0;
	b=p=status_message;
	
	while(True){
		if(!*p || '\n'==*p){
			Dimension tmp;

			/* height computation */
			XTextExtents(font, b, p-b, &dir, &ascent, &descent, &all);
			height+=(Dimension)(ascent+descent);

			/* width computation */
			tmp=(Dimension)XTextWidth(font, b, p-b);
			if(tmp>width) width=tmp; 

			b=p+1;
			if(!*p) break;
		}
		p++;
	}
	width+=WIDTHPADDING;
	height+=HEIGHTPADDING;

	/* 
  	 * now create the text widget, using the computed width and height and 
	 * the selection string.
	 */
	fm = CreateWidget("form", formWidgetClass, root, 
					   XtNdefaultDistance,	0,
					   NULL);
	button = CreateWidget("done", commandWidgetClass, fm,
					   XtNcallback, 	done,
					   XtNresizable,	False,
					   NULL);
	(void)CreateWidget("text", asciiTextWidgetClass, fm,
					   XtNwidth, 		width,
					   XtNheight, 		height,
					   XtNstring, 		status_message,
					   XtNfont, 		font,
					   XtNfromVert, 	button,
					   XtNborderWidth, 	0,
					   XtNresizable, 	True,
					   NULL); 

	XtRealizeWidget(root);
	XtMainLoop();
}

static void Xit(w, closure, call_data)
Widget w;
caddr_t closure;
caddr_t call_data;
{
	/* exit program */
	(void)exit(0);
}
